/* *************************************************************************
 *   Copyright (C) 2008 by Szymon Kulis (szymon.kulis@gmail.com)           *
 *                         http://freshlab.org/                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *                                                                         *
 *   Name         :  lcd.c                                                 *
 *   Description  :  This is a software driver for the graphic LCD with    * 
 *                   s1d10605 hardware driver.                             *
 *   Author       :  Szymon Kulis                                          *
 *   Remark       :  this work mostly is based on Nokia LCD Library        *
 *                   published by Sylvain Bissonnette on his webpage       *
 *                   (http://www.microsyl.com/)                            *
 *   History      :  2003-02-08 - First release                            * 
 *                                                                         *
 ************************************************************************* */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "types.h"
#include "compat.h"
#include "lcd.h"

//------------------------------------------------------------------------------
//                             Character generator
//      This table defines the standard ASCII characters in a 5x7 dot format.
//------------------------------------------------------------------------------
static const u08 FontLookup [][5] PROGMEM =
{
    { 0x00, 0x00, 0x00, 0x00, 0x00 },  // sp
    { 0x00, 0x00, 0x2f, 0x00, 0x00 },   // !
    { 0x00, 0x07, 0x00, 0x07, 0x00 },   // "
    { 0x14, 0x7f, 0x14, 0x7f, 0x14 },   // #
    { 0x24, 0x2a, 0x7f, 0x2a, 0x12 },   // $
    { 0xc4, 0xc8, 0x10, 0x26, 0x46 },   // %
    { 0x36, 0x49, 0x55, 0x22, 0x50 },   // &
    { 0x00, 0x05, 0x03, 0x00, 0x00 },   // '
    { 0x00, 0x1c, 0x22, 0x41, 0x00 },   // (
    { 0x00, 0x41, 0x22, 0x1c, 0x00 },   // )
    { 0x14, 0x08, 0x3E, 0x08, 0x14 },   // *
    { 0x08, 0x08, 0x3E, 0x08, 0x08 },   // +
    { 0x00, 0x00, 0x50, 0x30, 0x00 },   // ,
    { 0x10, 0x10, 0x10, 0x10, 0x10 },   // -
    { 0x00, 0x60, 0x60, 0x00, 0x00 },   // .
    { 0x20, 0x10, 0x08, 0x04, 0x02 },   // /
    { 0x3E, 0x51, 0x49, 0x45, 0x3E },   // 0
    { 0x00, 0x42, 0x7F, 0x40, 0x00 },   // 1
    { 0x42, 0x61, 0x51, 0x49, 0x46 },   // 2
    { 0x21, 0x41, 0x45, 0x4B, 0x31 },   // 3
    { 0x18, 0x14, 0x12, 0x7F, 0x10 },   // 4
    { 0x27, 0x45, 0x45, 0x45, 0x39 },   // 5
    { 0x3C, 0x4A, 0x49, 0x49, 0x30 },   // 6
    { 0x01, 0x71, 0x09, 0x05, 0x03 },   // 7
    { 0x36, 0x49, 0x49, 0x49, 0x36 },   // 8
    { 0x06, 0x49, 0x49, 0x29, 0x1E },   // 9
    { 0x00, 0x36, 0x36, 0x00, 0x00 },   // :
    { 0x00, 0x56, 0x36, 0x00, 0x00 },   // ;
    { 0x08, 0x14, 0x22, 0x41, 0x00 },   // <
    { 0x14, 0x14, 0x14, 0x14, 0x14 },   // =
    { 0x00, 0x41, 0x22, 0x14, 0x08 },   // >
    { 0x02, 0x01, 0x51, 0x09, 0x06 },   // ?
    { 0x32, 0x49, 0x59, 0x51, 0x3E },   // @
    { 0x7E, 0x11, 0x11, 0x11, 0x7E },   // A
    { 0x7F, 0x49, 0x49, 0x49, 0x36 },   // B
    { 0x3E, 0x41, 0x41, 0x41, 0x22 },   // C
    { 0x7F, 0x41, 0x41, 0x22, 0x1C },   // D
    { 0x7F, 0x49, 0x49, 0x49, 0x41 },   // E
    { 0x7F, 0x09, 0x09, 0x09, 0x01 },   // F
    { 0x3E, 0x41, 0x49, 0x49, 0x7A },   // G
    { 0x7F, 0x08, 0x08, 0x08, 0x7F },   // H
    { 0x00, 0x41, 0x7F, 0x41, 0x00 },   // I
    { 0x20, 0x40, 0x41, 0x3F, 0x01 },   // J
    { 0x7F, 0x08, 0x14, 0x22, 0x41 },   // K
    { 0x7F, 0x40, 0x40, 0x40, 0x40 },   // L
    { 0x7F, 0x02, 0x0C, 0x02, 0x7F },   // M
    { 0x7F, 0x04, 0x08, 0x10, 0x7F },   // N
    { 0x3E, 0x41, 0x41, 0x41, 0x3E },   // O
    { 0x7F, 0x09, 0x09, 0x09, 0x06 },   // P
    { 0x3E, 0x41, 0x51, 0x21, 0x5E },   // Q
    { 0x7F, 0x09, 0x19, 0x29, 0x46 },   // R
    { 0x46, 0x49, 0x49, 0x49, 0x31 },   // S
    { 0x01, 0x01, 0x7F, 0x01, 0x01 },   // T
    { 0x3F, 0x40, 0x40, 0x40, 0x3F },   // U
    { 0x1F, 0x20, 0x40, 0x20, 0x1F },   // V
    { 0x3F, 0x40, 0x38, 0x40, 0x3F },   // W
    { 0x63, 0x14, 0x08, 0x14, 0x63 },   // X
    { 0x07, 0x08, 0x70, 0x08, 0x07 },   // Y
    { 0x61, 0x51, 0x49, 0x45, 0x43 },   // Z
    { 0x00, 0x7F, 0x41, 0x41, 0x00 },   // [
    { 0x55, 0x2A, 0x55, 0x2A, 0x55 },   // 55
    { 0x00, 0x41, 0x41, 0x7F, 0x00 },   // ]
    { 0x04, 0x02, 0x01, 0x02, 0x04 },   // ^
    { 0x40, 0x40, 0x40, 0x40, 0x40 },   // _
    { 0x00, 0x01, 0x02, 0x04, 0x00 },   // '
    { 0x20, 0x54, 0x54, 0x54, 0x78 },   // a
    { 0x7F, 0x48, 0x44, 0x44, 0x38 },   // b
    { 0x38, 0x44, 0x44, 0x44, 0x20 },   // c
    { 0x38, 0x44, 0x44, 0x48, 0x7F },   // d
    { 0x38, 0x54, 0x54, 0x54, 0x18 },   // e
    { 0x08, 0x7E, 0x09, 0x01, 0x02 },   // f
    { 0x0C, 0x52, 0x52, 0x52, 0x3E },   // g
    { 0x7F, 0x08, 0x04, 0x04, 0x78 },   // h
    { 0x00, 0x44, 0x7D, 0x40, 0x00 },   // i
    { 0x20, 0x40, 0x44, 0x3D, 0x00 },   // j
    { 0x7F, 0x10, 0x28, 0x44, 0x00 },   // k
    { 0x00, 0x41, 0x7F, 0x40, 0x00 },   // l
    { 0x7C, 0x04, 0x18, 0x04, 0x78 },   // m
    { 0x7C, 0x08, 0x04, 0x04, 0x78 },   // n
    { 0x38, 0x44, 0x44, 0x44, 0x38 },   // o
    { 0x7C, 0x14, 0x14, 0x14, 0x08 },   // p
    { 0x08, 0x14, 0x14, 0x18, 0x7C },   // q
    { 0x7C, 0x08, 0x04, 0x04, 0x08 },   // r
    { 0x48, 0x54, 0x54, 0x54, 0x20 },   // s
    { 0x04, 0x3F, 0x44, 0x40, 0x20 },   // t
    { 0x3C, 0x40, 0x40, 0x20, 0x7C },   // u
    { 0x1C, 0x20, 0x40, 0x20, 0x1C },   // v
    { 0x3C, 0x40, 0x30, 0x40, 0x3C },   // w
    { 0x44, 0x28, 0x10, 0x28, 0x44 },   // x
    { 0x0C, 0x50, 0x50, 0x50, 0x3C },   // y
    { 0x44, 0x64, 0x54, 0x4C, 0x44 }    // z
};

//------------------------------------------------------------------------------
//                              Global Variables
//------------------------------------------------------------------------------
static u08 LcdCache [ LCD_CACHE_SIZE ];
static u16 LcdCacheIdx;
static u08 lcd_x;
static u08 lcd_y;

void LcdSend ( u08 data, LcdCmdData cd )
{
  u08 i;
  //data or command 
  if ( cd == LCD_DATA )
    sbi(DC_PORT,DC_PIN);
  else
    cbi(DC_PORT,DC_PIN);

  //shift 8 bits
  for(i=0;i<8;i++)
  {
   cbi(SCL_PORT,SCL_PIN);
   if(data&(0x80)) 
     sbi(SDAT_PORT,SDAT_PIN);
   else
     cbi(SDAT_PORT,SDAT_PIN);
   data<<=1;
   sbi(SCL_PORT,SCL_PIN);
  }
}

void LcdInit()
{
  //set/clear pins
  cbi(RES_PORT,RES_PIN);
  cbi(CS_PORT,CS_PIN);
  cbi(DC_PORT,DC_PIN);
  sbi(SCL_PORT,SCL_PIN);
  cbi(SDAT_PORT,SDAT_PIN);

  //set pins as outpus
  sbi(RES_DDR,RES_PIN);
  sbi(CS_DDR,CS_PIN);
  sbi(DC_DDR,DC_PIN);
  sbi(SCL_DDR,SCL_PIN);
  sbi(SDAT_DDR,SDAT_PIN);

  //after reset  
  asm("nop");
  sbi(RES_PORT,RES_PIN);
  asm("nop");

  //configure lcd registers
  LcdSend(0xE2, LCD_CMD);       // Reset
  LcdSend(0xE3, LCD_CMD);       // NOP
  LcdSend(0xE2, LCD_CMD);       // Reset
  LcdSend(0xA1, LCD_CMD);	// ADC=1
  LcdSend(0xC8, LCD_CMD);	// Common Output Mode Select
  LcdSend(0x25, LCD_CMD);       // Resistor ratio
  LcdSend(0x81, LCD_CMD);       // electronic volume
  LcdSend(0x19, LCD_CMD);       // electronic volume value;
  LcdSend(0x28+7, LCD_CMD);	// power con
  LcdSend(0x40, LCD_CMD);	// start line
  LcdSend(0xA7, LCD_CMD);	// Display normal
  LcdSend(0xAC, LCD_CMD);
  LcdSend(0x03, LCD_CMD); 
  LcdSend(0xA4, LCD_CMD);	// all dots off
  LcdSend(0xAF, LCD_CMD);	// Display on
  
  //clear bufer
  LcdClear();
  //update lcd ram
  LcdUpdate();
}


void LcdContrast ( u08 contrast )
{
  LcdSend(0x81, LCD_CMD);       // electronic volume
  LcdSend(contrast, LCD_CMD);   // electronic volume value;
}


void LcdClear ( void )
{
  u16 i;
  for ( i = 0; i < LCD_CACHE_SIZE; i++ )
    LcdCache[i] = 0x00;
  lcd_x=0;
  lcd_y=0;
}


void LcdGotoXY ( u08 x, u08 y )
{
   lcd_x=x;
   lcd_y=y;
//   LcdCacheIdx = (x - 1) * 6 + (y - 1) * LCD_X_RES;
}


void LcdChr ( LcdFontSize size, u08 ch )
{
  u08 i, c;
  u08 b1, b2;
  u16 tmpIdx;
  u08 y0;
  u08 offset;

  //  Convert to a printable character.
  if ( (ch < 0x20) || (ch > 0x7b) )
    ch = 92;

  y0=(lcd_y/8)*8;
  LcdCacheIdx=(lcd_y/8)*LCD_X_RES + lcd_x;
  offset=lcd_y-y0;
  
  // FONT_1X
  if ( size == FONT_1X )
  {
    if(offset==0)
      for ( i = 0; i < 5; i++ )
      {
        b1=pgm_read_byte(&FontLookup[ch - 32][i]);
        LcdCache[LcdCacheIdx] |=  b1 ;
        LcdCacheIdx++;
      }
    else // offset!=0
      for ( i = 0; i < 5; i++ )
      {
        b1=pgm_read_byte(&FontLookup[ch - 32][i]);
        LcdCache[LcdCacheIdx] |=  b1 << (1+offset);
        LcdCache[LcdCacheIdx+LCD_X_RES] |= b1  >> (7-offset);
        LcdCacheIdx++;
      }
    lcd_x+=6;
  }
  // FONT_2X
  else if ( size == FONT_2X )
  {

    tmpIdx = LcdCacheIdx;

    if(offset==0)
    {
      for ( i = 0; i < 5; i++ )
      {
        c = pgm_read_byte(&FontLookup[ch - 32][i]) << 1;
        b1 =  (c & 0x01) * 3;
        b1 |= (c & 0x02) * 6;
        b1 |= (c & 0x04) * 12;
        b1 |= (c & 0x08) * 24;

        c >>= 4;
        b2 =  (c & 0x01) * 3;
        b2 |= (c & 0x02) * 6;
        b2 |= (c & 0x04) * 12;
        b2 |= (c & 0x08) * 24;

        LcdCache[tmpIdx++] = b1;
        LcdCache[tmpIdx++] = b1;
        LcdCache[tmpIdx + LCD_X_RES -2] = b2;
        LcdCache[tmpIdx + LCD_X_RES - 1] = b2;
      }
      lcd_x += 11;
    } 
    else // offset!=0
    {
      for ( i = 0; i < 5; i++ )
      {
        c = pgm_read_byte(&FontLookup[ch - 32][i]) << 1;
        b1 =  (c & 0x01) * 3;
        b1 |= (c & 0x02) * 6;
        b1 |= (c & 0x04) * 12;
        b1 |= (c & 0x08) * 24;

        c >>= 4;
        b2 =  (c & 0x01) * 3;
        b2 |= (c & 0x02) * 6;
        b2 |= (c & 0x04) * 12;
        b2 |= (c & 0x08) * 24;

        LcdCache[tmpIdx++] |=  b1 << (1+offset);
        LcdCache[tmpIdx++] |=  b1 << (1+offset);
        LcdCache[tmpIdx + LCD_X_RES -2]   |= ( b1  >> (7-offset) ) + ( b2 << (1+offset) );
        LcdCache[tmpIdx + LCD_X_RES -1]   |= ( b1  >> (7-offset) ) + ( b2 << (1+offset) );
        LcdCache[tmpIdx + 2*LCD_X_RES -2] |= b2  >> (7-offset);
        LcdCache[tmpIdx + 2*LCD_X_RES -1] |= b2  >> (7-offset);
      }
      lcd_x += 12;
    } 
  }
}




void LcdStr ( LcdFontSize size, u08 *dataPtr )
{
  while ( *dataPtr )
    LcdChr( size, *dataPtr++ );
}


void LcdPixel ( u08 x, u08 y, LcdPixelMode mode )
{
    u16  index;
    u08  offset;
    u08  data;

    if ( x > LCD_X_RES ) return;
    if ( y > LCD_Y_RES ) return;

    index = ((y / 8) * LCD_X_RES) + x;
    offset  = y - ((y / 8) * 8);

    data = LcdCache[index];

    if ( mode == PIXEL_OFF )
    {
        data &= (~(0x01 << offset));
    }
    else if ( mode == PIXEL_ON )
    {
        data |= (0x01 << offset);
    }
    else if ( mode  == PIXEL_XOR )
    {
        data ^= (0x01 << offset);
    }

    LcdCache[index] = data;

}

void LcdHLine ( u08 x1, u08 y1, u08 x2, LcdPixelMode mode )
{
 //swap
 if(x1>x2)
 {
   u08 tmp=x1;
   x1=x2;
   x2=tmp;
 }
 while(x1<=x2)
   LcdPixel( x1++, y1, mode );
}

void LcdVLine ( u08 x1, u08 y1, u08 y2, LcdPixelMode mode )
{
 //swap
 if(y1>y2)
 {
   u08 tmp=y1;
   y1=y2;
   y2=tmp;
 }
 while(y1<=y2)
   LcdPixel( x1, y1++, mode );
}


void LcdLine ( u08 x1, u08 y1, u08 x2, u08 y2, LcdPixelMode mode )
{
    s08 dx, dy, stepx, stepy, fraction;

    dy = y2 - y1;
    dx = x2 - x1;

    if ( dy < 0 )
    {
        dy    = -dy;
        stepy = -1;
    }
    else
    {
        stepy = 1;
    }

    if ( dx < 0 )
    {
        dx    = -dx;
        stepx = -1;
    }
    else
    {
        stepx = 1;
    }

    dx <<= 1;
    dy <<= 1;

    LcdPixel( x1, y1, mode );

    if ( dx > dy )
    {
        fraction = dy - (dx >> 1);
        while ( x1 != x2 )
        {
            if ( fraction >= 0 )
            {
                y1 += stepy;
                fraction -= dx;
            }
            x1 += stepx;
            fraction += dy;
            LcdPixel( x1, y1, mode );
        }
    }
    else
    {
        fraction = dx - (dy >> 1);
        while ( y1 != y2 )
        {
            if ( fraction >= 0 )
            {
                x1 += stepx;
                fraction -= dy;
            }
            y1 += stepy;
            fraction += dx;
            LcdPixel( x1, y1, mode );
        }
    }
}

void LcdUpdate ( void )
{
  u08 i;
  u08 page;
  for(page=0;page<LCD_Y_RES/8;page++)
  {
    LcdSend(0x40, LCD_CMD);            //start line
    LcdSend(0xB0+(page&0x0F), LCD_CMD);//page
    LcdSend(0x10, LCD_CMD);            //column hi
    LcdSend(0x00, LCD_CMD);            //column lo
    for(i=0;i<LCD_X_RES;i++)
      LcdSend( LcdCache[page*LCD_X_RES + i], LCD_DATA );
  }
}
void LcdRectangle ( u08 x1, u08 y1, u08 x2, u08 y2, LcdPixelMode mode )
{
 //swap
 if(x1>x2)
 {
   u08 tmp=x1;
   x1=x2;
   x2=tmp;
 }
 //swap
 if(y1>y2)
 {
   u08 tmp=y1;
   y1=y2;
   y2=tmp;
 }

 u08 x=x1,y=y1+1;
 while(x<=x2)
 {
   LcdPixel( x,   y1, mode );
   LcdPixel( x++, y2, mode );
 }
 while(y<=y2-1)
 {
   LcdPixel( x1, y  , mode );
   LcdPixel( x2, y++, mode );
 }
}

void LcdBox ( u08 x1, u08 y1, u08 x2, u08 y2, LcdPixelMode mode )
{
 //swap
 if(x1>x2)
 {
   u08 tmp=x1;
   x1=x2;
   x2=tmp;
 }
 //swap
 if(y1>y2)
 {
   u08 tmp=y1;
   y1=y2;
   y2=tmp;
 }

  u08 x,y;
  for(x=x1;x<=x2;x++)
    for(y=y1;y<=y2;y++)
      LcdPixel(x,y,mode);
}


