/* *************************************************************************
 *   Copyright (C) 2008 by Szymon Kulis (szymon.kulis@gmail.com)           *
 *                         http://freshlab.org/                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 *                                                                         *
 *                                                                         *
 *   Name         :  lcd.h                                                 *
 *   Description  :  This is a software driver for the graphic LCD with    * 
 *                   s1d10605 hardware driver.                             *
 *   Author       :  Szymon Kulis                                          *
 *   Remark       :  this work mostly is based on Nokia LCD Library        *
 *                   published by Sylvain Bissonnette on his webpage       *
 *                   (http://www.microsyl.com/)                            *
 *   History      :  2003-02-08 - First release                            * 
 *                                                                         *
 ************************************************************************* */


#ifndef _LCD_H_
#define _LCD_H_

#include "types.h"
#include "lcd_conf.h"

//------------------------------------------------------------------------------
//                           Type definitions
//------------------------------------------------------------------------------

/**
 * Determine data or command to be sent
 */
typedef enum
{
    LCD_CMD  = 0, /**< Send command */ 
    LCD_DATA = 1  /**< Send data */ 

} LcdCmdData;

/**
 * Pixel drawing mode
 */
typedef enum
{
    PIXEL_OFF =  0, /**< Switch off pixel */ 
    PIXEL_ON  =  1, /**< Switch on pixel */ 
    PIXEL_XOR =  2  /**< Xor pixel */ 

} LcdPixelMode;

/**
 * Font size
 */
typedef enum
{
    FONT_1X = 1, /**< Normal size (8 pixels)*/ 
    FONT_2X = 2  /**< Double size (16 pixels)*/ 

} LcdFontSize;

//------------------------------------------------------------------------------
//                             Public function prototypes
//------------------------------------------------------------------------------


/**
 * Performs MCU pins & LCD controller initialization.
 */
void LcdInit ( void );

/**
 * Sends data to display controller.
 * @param data Data to be sent
 * @param cd   Command or data
 * @see LcdCmdData
 */
void LcdSend ( u08 data, LcdCmdData cd );

/**
 * Set Lcd display contrast.
 * @param contrast Contrast value from 0x00 to 0x2F.
 */
void LcdContrast ( u08 contrast );

/**
 * Clears the display's buffor. LcdUpdate must be called next.
 */
void LcdClear ( void );

/**
 * Copies the LCD cache into the device RAM. Have to be called 
 * each time somthing was changed.                  
 */
void LcdUpdate ( void );

/**
 * Sets cursor location to xy location corresponding to basic font size.
 * @param x Coordinate for new cursor position. Range: [0,LCD_X_RES-1]
 * @param y Coordinate for new cursor position. Range: [0,LCD_Y_RES-1]  
 */
void LcdGotoXY ( u08 x, u08 y );

/**
 * Displays a character at current cursor location and increment cursor location.
 * @param  size  Font size. See enum.
 * @param  ch    Character to write.
 * @see LcdFontSize LcdStr
 */
void LcdChr ( LcdFontSize size, u08 ch );

/**
 * Displays a character at current cursor location and increment cursor location
 * according to font size.
 * @param size Font size.
 * @param dataPtr Pointer to null terminated ASCII string to display.
 * @see LcdFontSize LcdChr
 */
void LcdStr ( LcdFontSize size, u08 *dataPtr );

/**
 * Displays a pixel at given absolute (x, y) location.
 * @param x, y Absolute pixel coordinates
 * @param mode Off, On or Xor.
 * @see LcdPixelMode
 */
void LcdPixel ( u08 x, u08 y, LcdPixelMode mode );

/**
 * Draws a line between two points on the display.
 * @param x1, y1 Absolute pixel coordinates for line origin.
 * @param x2, y2 Absolute pixel coordinates for line end.
 * @param mode   Off, On or Xor. 
 * @see LcdPixelMode LcdHLine LcdVLine
 */
void LcdLine ( u08 x1, u08 y1, u08 x2, u08 y2, LcdPixelMode mode );

/**
 * Draws a horizontal line between two points on the display.
 * @param x1, y1 Absolute pixel coordinates for line origin.
 * @param x2     Absolute pixel coordinates for line end. (y2=y1)
 * @param  mode  Off, On or Xor.
 * @see LcdPixelMode LcdLine LcdVLine
 */
void LcdHLine ( u08 x1, u08 y1, u08 x2, LcdPixelMode mode );

/**
 * Draws a vertical line between two points on the display.
 * @param x1, y1 Absolute pixel coordinates for line origin.
 * @param y2     Absolute pixel coordinates for line end. (x2=x1)
 * @param mode   Off, On or Xor. 
 * @see LcdPixelMode LcdLine LcdHLine
 */
void LcdVLine ( u08 x1, u08 y1, u08 y2, LcdPixelMode mode );

/**
 * Draws a rectangle with the given top left corner, 
 * and right bottom corner.
 * @param x1, y1  Absolute pixel coordinates for left top corner.
 * @param x2, y2  Absolute pixel coordinates for right bottom corner.
 * @param mode    Off, On or Xor.
 * @param None.
 * @see LcdPixelMode LcdBox
 */
void LcdRectangle ( u08 x1, u08 y1, u08 x2, u08 y2, LcdPixelMode mode );

/**
 * Draws a field rectangle with the given top left corner, 
 * and right bottom corner.
 * @param  x1, y1 Absolute pixel coordinates for line origin.
 * @param  x2, y2 Absolute pixel coordinates for line end.
 * @param mode    Off, On or Xor.
 * @see LcdPixelMode LcdRectangle
 */
void LcdBox ( u08 x1, u08 y1, u08 x2, u08 y2, LcdPixelMode mode );
#endif  //  _LCD_H_

